// discretize angles into bins
// function splitOrientations(mag,ang,thresh,fullcircle,nbins)
// mag - magnitudes - are splited proportionaly into 2 bins closest to the angle; if < 0 - point doesn't exist
// angles - angles corresponding to magnitudes, expeted to be in range 0-2pi
// thresh - mag > 0 && mag < thresh are stored in additional bin at the end
// fullcircle - if false then only range 0-180 is divided into bins, angles(angles > 180) = angles - 180
// nbins - number of bins
// magnitude do not influence voting !!!!!!!!!!!
#include "mex.h"
#include "matrix.h" // mxArray definition
#include <math.h>  // floor()

#if defined(NAN_EQUALS_ZERO)
#define IsNonZero(d) ((d)!=0.0 || mxIsNaN(d))
#else
#define IsNonZero(d) ((d)!=0.0)
#endif

void mexFunction(
		 int nlhs,       mxArray *plhs[],
		 int nrhs, const mxArray *prhs[]
		 )
{
   
    /* Check for proper number of input and output arguments */    
    if (!(nrhs == 5))
      mexErrMsgTxt("5 input arguments required - mag,ang,thresh,fullcircle,nbins");
    if(nlhs > 3)
      mexErrMsgTxt("Too many output arguments.");
    
    /* Check data type of input argument  */
    if (!(mxIsDouble(prhs[0])))
      mexErrMsgTxt("magnitudes must be of type double.");
    if (!(mxIsDouble(prhs[1])))
      mexErrMsgTxt("angles must be of type double.");
    if (!(mxIsDouble(prhs[2])))
      mexErrMsgTxt("thresh must be of type double.");
    if (!(mxIsLogical(prhs[3])))
      mexErrMsgTxt("fullcircle must be of type logical.");
    if (!(mxIsUint32(prhs[4])))
      mexErrMsgTxt("nbins must be of type uint32.");    
    
    double thresh = mxGetScalar(prhs[2]);
    bool fullcircle = (bool)mxGetScalar(prhs[3]);
    int nbins = (int)mxGetScalar(prhs[4]);
    
    mwSize dims[3];
    dims[2] = nbins;
    
    // handle magnitude data
    dims[0] = mxGetM(prhs[0]);
    dims[1] = mxGetN(prhs[0]);
    long N = (long)dims[0]*dims[1];
    double* mag = mxGetPr(prhs[0]);
        
    // handle angle data
    mwSize Nang  = mxGetM(prhs[1]);
    Nang *= mxGetN(prhs[0]);
    double* ang = mxGetPr(prhs[1]);
    
    if (N != Nang)
      mexErrMsgTxt("mag & angle dimensions do not agree");
    
    double range;
       
    if(fullcircle == false)
      range = M_PI;
    else 
      range = 2*M_PI;

    // compute bin centers
    double sizbin = range/nbins;
    mxArray* binCentres = mxCreateDoubleMatrix(nbins,1,mxREAL);
    double* binC = mxGetPr(binCentres);
    for(int j=0; j<nbins; j++)
      binC[j] = (j)*sizbin;

    
    // allocate output 
    // functions mxCreate set the allocated mem to 0, no need of memset
    mxArray* orientations = mxCreateNumericArray(3,dims,mxDOUBLE_CLASS,mxREAL);
    double* orient = mxGetPr(orientations);
    mxArray* stillbin = mxCreateDoubleMatrix(dims[0],dims[1],mxREAL);
    double* still = mxGetPr(stillbin);
 
    double currangle;
    double proportion;
    double temp;
    int lower;
    int upper;
    unsigned long p;
    // loop through all pixels 
    for(p=0; p < N; p++)
    {
      if(mxIsNaN(mag[p]))
        continue; //when magnitude is nan then there is no feature present in this point)
      if(mag[p] < thresh)
        still[p] = 1;
      else
      {
        if(!fullcircle && ang[p] > M_PI)
          currangle = ang[p]-M_PI;
        else
          currangle = ang[p];
          temp = currangle/sizbin;
          lower = (int)floor(temp);
          proportion = temp-lower;
          lower = lower % nbins;
          upper = (lower+1) % nbins;
          orient[p+N*lower] = (1-proportion);
          orient[p+N*upper] = proportion;
      }
    }
        
  // assign created orient and still to outputs
  plhs[0] = orientations;   
  plhs[1] = stillbin;
  plhs[2] = binCentres;
}
 

