#include "math.h"
#include "mex.h"
#include <iostream>
#define VAR(x)  #x " = " << x 

#undef max
#undef min

#include <limits>

using namespace std;
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
        // parameters:
        // - features
        // - centroids

        // output:
        // - cluster ids


        const mxArray *mxFeatures=prhs[0],
                *mxCentroids=prhs[1];

        double *feat=mxGetPr(mxFeatures);
        double *centroids=mxGetPr(mxCentroids);

        int N=mxGetN(mxFeatures);
        int D=mxGetM(mxFeatures);
        int C=mxGetN(mxCentroids);
        int D2=mxGetM(mxCentroids);

        //mexPrintf("Assigning %i %i-dimensional features to %i clusters\n",N,D,C);
        
        if(D!=D2) {
                mexErrMsgTxt("centroids and features have different dimensionalities.");
        }
        

        plhs[0]=mxCreateNumericMatrix(1,N,mxINT32_CLASS,0);
        int *clusterIds=(int*)mxGetData(plhs[0]);

        double tmp;
        for(uint n=0;n<N;++n) {
                double minDist=std::numeric_limits<double>::max();
                int bestCluster=-1;
                
                for(uint c=0;c<C;++c) {
                        double curDist=0.0;
                        for(uint d=0;d<D;++d) {
                                tmp=feat[n*D+d]-centroids[c*D+d];
                                curDist+=tmp*tmp;
                        }
                        if(curDist<minDist) {
                                minDist=curDist;
                                bestCluster=c;
                        }
                }
                clusterIds[n]=bestCluster+1;
        }
        
}

