function matrix_features = classcutMatrixFeatures(obj,d,dir_extract_features,image_type,kmeans_nb,grid_dist,option_features,quantizier)

% for directory d process all image with type image_type
%processing: for every image read the segmentation and associate a vector
%to every superpixel matrix_features{image}(feature,superpixel) normalized

org_dir = pwd;
org_ldlp = getenv('LD_LIBRARY_PATH');
setenv('LD_LIBRARY_PATH', '');
cd(d);
images = dir(['*.' image_type]);

current_image = 0;

if(strfind(option_features,'surf'))
  %extract SURF features
  fname = ['matrix_features_surf_' num2str(kmeans_nb) '_' num2str(grid_dist) '.mat'];
  settings_surf = as_GetSurfExtractionSettings();
  settings_surf.surf.kmeans_k = kmeans_nb;
  
  allfeat=[];
  
  cd(dir_extract_features);
  images1 = dir(['*.' image_type]);
  try
      struct = load('allfeat.mat');
      allfeat = struct.allfeat;
  catch
      for i = images1'
          img = im2double(rgb2gray(gray2rgb(imread([dir_extract_features '/' i.name]))));
          imgsize = size(img);
          f = surfDes(img,gengrid(imgsize(2),imgsize(1),[0.01 0.02 0.05],grid_dist));
          allfeat = [allfeat; f];
          %i.name
          %size(allfeat)
      end
      try      
          save('allfeat.mat','allfeat','-v7');          
      catch
          save('allfeat.mat','allfeat','-v7.3');   
      end
  end
  cd(d);
  nb_feat = size(allfeat,1);
  permutation = randperm(nb_feat);
  allfeat = allfeat(permutation(1:round(nb_feat/10)),:);
  size(allfeat)
  %clustering using vgg matlab
  %opts = struct('maxiters', inf, 'mindelta', eps, 'verbose', 1); 
  [CX,sse]=vgg_kmeans(allfeat',settings_surf.surf.kmeans_k,'maxiters',100,'mindelta',1,'verbose',1);

  settings_surf.surf.CX = CX;%cluster centroids
  save(['CX_' num2str(kmeans_nb)],'CX');

  for i = images' 
    current_image = current_image + 1;
    name = i.name;
    i_base = name(1:(length(name)-length(image_type)-1));
    img = im2double(rgb2gray(gray2rgb(imread([d '/' name]))));
    imgsize = size(img);
 
    matrix_features{current_image} = zeros(settings_surf.surf.kmeans_k, length(obj(current_image).segms)); % cell coresp to features for current image
    for j = 1: length(obj(current_image).segms)
        pts =[];
        %compute interest point inside each superpixel
        for point = 1 : size(obj(current_image).segms(j).coords,2)
            x = obj(current_image).segms(j).coords(1,point);        
            y = obj(current_image).segms(j).coords(2,point);
            if((mod(x,grid_dist)==0) && (mod(y,grid_dist)==0)) %check whether (x,y) is an interest point
                x = double(x);
                y = double(y);
                pts = [pts;x y 0.01 0 0.01;x y 0.02 0 0.02;x y 0.05 0 0.05];                
            end
        end
       
        if(numel(pts))
            
            if(pts(1,5))            
                feat_segm = surfDes(img,pts);
            end

            viswords = quantise(feat_segm',settings_surf.surf.CX);
            count_vector = zeros(settings_surf.surf.kmeans_k,1);
            for nb_centroid=1:settings_surf.surf.kmeans_k
                count_vector(nb_centroid) = length(find(viswords == nb_centroid));
            end
            matrix_features{current_image}(:,j) = count_vector/sum(count_vector);
        end
    end
    
  end
else
    if(strcmpi(option_features,'ch'))
        %quantizier = param.quantizier;        
        option='rgb';%'lab'
        for i = images'
            current_image = current_image + 1;
            name = i.name;
            i_base = name(1:(length(name)-length(image_type)-1));
            img = imread([d '/' name]);
            img = double(img);            
            
            matrix_features{current_image} = zeros(sum(prod(quantizier')), length(obj(current_image).segms)); % cell coresp to features for current image
                                       
            %MARCIN's triLinearVoting for the original image
            for j = 1: length(obj(current_image).segms) % nb_of_superpixels in current image = length(obj{current_image}.segms)
                img_superpixel = zeros(size(obj(current_image).segms(j).coords,2),1,3);
                nb_pixels = 0;
                for point = 1 : size(obj(current_image).segms(j).coords,2)
                    nb_pixels = nb_pixels + 1;
                    x = obj(current_image).segms(j).coords(1,point);
                    y = obj(current_image).segms(j).coords(2,point);
                    img_superpixel(nb_pixels,1,:) = img(y,x,:);
                end
                
                produs = prod(quantizier');%quantizier was in the old implementation [quantizier1 ; quantizier2; quantizier3; ... ]
                position = 0;
                for line = 1:size(quantizier,1)
                    temp = triLinearVoting(reshape(double(img_superpixel),size(img_superpixel,1),3),quantizier(line,:));
                    matrix_features{current_image}(position+1:position+produs(line),j) = temp;%unnormalized histo
                    %matrix_features{current_image}(position+1:position+produs(line),j) = temp/sum(temp);
                    position = position + produs(line);
                end
                %matrix_features{current_image}(:,j) = triLinearVoting(reshape(double(img_superpixel),size(img_superpixel,1),3),quantizier);
                %matrix_features{current_image}(:,j) = matrix_features{current_image}(:,j)/ sum(matrix_features{current_image}(:,j));%normalizing the feature vector
            end
        end        
        fname = ['matrix_features_ch_rgb_' num2str(quantizier(1)) '_' num2str(quantizier(2)) '_' num2str(quantizier(3))];
    end
end

save(fname,'matrix_features')

cd(org_dir);
setenv('LD_LIBRARY_PATH', org_ldlp);
end