function paramsClassCut = classcutPreprocessingStage(dir_root,inputDir,imageType,paramsClassCut)

%run preprocessing stage for ClassCut on all images in the inputDir

origDir = pwd;
org_ldlp = getenv('LD_LIBRARY_PATH');
setenv('LD_LIBRARY_PATH', '');


%paramsClassCut = classcutDefaultParams(paramsClassCut);

paramsClassCut.inputDir = inputDir;
paramsClassCut.imageType = imageType;

dirCroppedBBoxImages = 'CroppedBBoxImages';
dirCroppedBBoxNormMedianImages = 'CroppedBBoxNormMedianImages';
dirResizedOrigImages = 'ResizedOriginalImages';

%load the soft_mask
if exist([inputDir '/soft_mask_lfc.mat'],'file')
    structTemp = load([inputDir '/soft_mask_lfc.mat']);
    paramsClassCut.mask = structTemp.soft_mask;
else
    if exist([inputDir '/' dirCroppedBBoxNormMedianImages  '/soft_mask_lfc.mat'],'file')
        structTemp = load([inputDir '/' dirCroppedBBoxNormMedianImages  '/soft_mask_lfc.mat']);
        paramsClassCut.mask = structTemp.soft_mask;
    else
        structTemp = load([dir_root '/soft_mask/soft_mask_lfc.mat']);
        paramsClassCut.mask = structTemp.soft_mask;
    end
end

%derive the coordinate frame
classcutDeriveCoordinateFrame(inputDir,imageType,paramsClassCut);
if paramsClassCut.verbose; disp('Deriving Coord Frame finished'); end;


%derive the CroppedBBoxImages
cd(inputDir);
if ~isdir(dirCroppedBBoxImages)
    display(['Creating ' dirCroppedBBoxImages]);
    mkdir(dirCroppedBBoxImages);   
end
classcutGetCroppedImages(inputDir,[inputDir '/' dirCroppedBBoxImages '/'],imageType);
if paramsClassCut.verbose; disp('Deriving Cropped Images finished'); end;


%derive the CroppedBBoxNormMedianImages
if ~isdir(dirCroppedBBoxNormMedianImages)
    display(['Creating ' dirCroppedBBoxNormMedianImages]);
    mkdir(dirCroppedBBoxNormMedianImages);    
end
classcutGetNormMedianImages([inputDir '/' dirCroppedBBoxImages '/'],[inputDir '/' dirCroppedBBoxNormMedianImages '/'],imageType);
if paramsClassCut.verbose; disp('Deriving CroppedNorm Images finished'); end;

paramsClassCut.workingDir = [inputDir '/' dirCroppedBBoxNormMedianImages '/'];


%resize the orig images for feature extraction
if ~isdir(dirResizedOrigImages)
    display(['Creating ' dirResizedOrigImages]);
    mkdir(dirResizedOrigImages);
end
classcutGetResizeOriginalImagesProportionalCroppedBBox(inputDir,[inputDir '/' dirResizedOrigImages '/'],[inputDir '/' dirCroppedBBoxImages '/'],[inputDir '/' dirCroppedBBoxNormMedianImages '/'],imageType);
if paramsClassCut.verbose; disp('Deriving ResizeOriginalImagesProportionalCroppedBBox finished'); end;

paramsClassCut.full_images_resized = [inputDir '/' dirResizedOrigImages '/'];


%get the obj struct
if not(exist([inputDir '/' dirCroppedBBoxNormMedianImages '/obj.mat'],'file'))
    if paramsClassCut.verbose; disp('Creating OBJ');end
    classcutSegmentImagesDir(paramsClassCut.soft_dir,[inputDir '/' dirCroppedBBoxNormMedianImages '/'],imageType, paramsClassCut.segm_params);
else
    if paramsClassCut.verbose; disp('OBJ exists');end   
end

cd([inputDir '/' dirCroppedBBoxNormMedianImages '/']); 

%get the surf BOW - 100
if not(exist([inputDir '/' dirCroppedBBoxNormMedianImages '/matrix_features_surf_100_4.mat'],'file'))
    if paramsClassCut.verbose; disp('Creating matrix_features_surf_100_4');end
    structTemp = load('obj.mat');
    obj = structTemp.obj;
    classcutMatrixFeatures(obj,[inputDir '/' dirCroppedBBoxNormMedianImages '/'],[inputDir '/' dirResizedOrigImages '/'],imageType,100,4,'surf',[]);    
else
    if paramsClassCut.verbose; 
        display('matrix_features_surf_100_4 exists');
    end
end


%get the surf BOW - 1000
if not(exist([inputDir '/' dirCroppedBBoxNormMedianImages '/matrix_features_surf_1000_4.mat'],'file'))
    if paramsClassCut.verbose; disp('Creating matrix_features_surf_1000_4');end
    structTemp = load('obj.mat');
    obj = structTemp.obj;
    classcutMatrixFeatures(obj,[inputDir '/' dirCroppedBBoxNormMedianImages '/'],[inputDir '/' dirResizedOrigImages '/'],imageType,1000,4,'surf',[]);
else
    if paramsClassCut.verbose; disp('matrix_features_surf_1000_4 exists');end
end

%get the obj struct used to derive the segm at the first iteration
if not(exist([inputDir '/' dirCroppedBBoxNormMedianImages '/obj_init_soft_mask_lfc.mat'],'file'))
    if paramsClassCut.verbose; disp('Creating OBJ_INIT_SOFT_MASK_LFC');end
    classcutGetDataPreprocessedSoftMaskFromClasses(paramsClassCut);
else
    if paramsClassCut.verbose; disp('OBJ_INIT_SOFT_MASK_LFC exists');end
end


%get the bg model outside the coordinate frame - surf BOW - 100
if not(exist([inputDir '/' dirCroppedBBoxNormMedianImages '/ModelBGOutsideBox_surf_100.mat'],'file'))
    if paramsClassCut.verbose; disp('Creating ModelBGOutsideBox_surf_100.mat');end
    classcutGetModelBGOutsideBox(paramsClassCut,'surf',100);
else
    if paramsClassCut.verbose; disp('ModelBGOutsideBox_surf_100.mat exists');end
end


%get the bg model outside the coordinate frame - surf BOW - 1000
if not(exist([inputDir '/' dirCroppedBBoxNormMedianImages '/ModelBGOutsideBox_surf_1000.mat'],'file'))
    if paramsClassCut.verbose; disp('Creating ModelBGOutsideBox_surf_1000.mat');end
    classcutGetModelBGOutsideBox(paramsClassCut,'surf',1000);
else
    if paramsClassCut.verbose; disp('ModelBGOutsideBox_surf_1000.mat exists');end
end


%get the graphical models

%within image smoothness
if not(exist([inputDir '/' dirCroppedBBoxNormMedianImages '/GradientSmth.mat'],'file'))
    if paramsClassCut.verbose; disp('Creating GradientSmth GM');end
    structTemp = load('obj.mat');
    obj = structTemp.obj;
    GM = classcutGetPairwiseSmoothnessGradientReduced(obj,1);
    save([inputDir '/' dirCroppedBBoxNormMedianImages '/GradientSmth.mat'],'GM');
else
    if paramsClassCut.verbose; disp('GradientSmth GM exists');end
end


%between image smoothness
if not(exist([inputDir '/' dirCroppedBBoxNormMedianImages '/LocationSmth.mat'],'file'))
    if paramsClassCut.verbose; disp('Creating LocationSmth GM');end
    structTemp = load('obj.mat');
    obj = structTemp.obj;
    GM = classcutGetPairwiseLocationSmoothnessImagesReduced(obj);
    save([inputDir '/' dirCroppedBBoxNormMedianImages '/LocationSmth.mat'],'GM');
else
    if paramsClassCut.verbose; disp('LocationSmth GM exists');end
end


%border
if not(exist([inputDir '/' dirCroppedBBoxNormMedianImages '/GM_Border.mat'],'file'))
    if paramsClassCut.verbose; disp('Creating GM_Border');end
    structTemp = load('obj.mat');
    obj = structTemp.obj;
    GM = classcutGetUnaryBorder(obj,'soft'); 
    save('GM_Border','GM');
else
    if paramsClassCut.verbose; disp('GM_border exists');end
end


%area
if not(exist([inputDir '/' dirCroppedBBoxNormMedianImages '/GM_Area.mat'],'file'))
    if paramsClassCut.verbose; disp('Creating GM_Area');end
    structTemp = load('obj.mat');
    obj = structTemp.obj;
    GM = classcutGetUnaryArea(obj);
    save('GM_Area','GM');
else
    if paramsClassCut.verbose; disp('GM_Area exists');end
end


%get the inital shape
if not(exist([inputDir '/' dirCroppedBBoxNormMedianImages '/global_shape_init_soft.mat'],'file'))
    if paramsClassCut.verbose; disp('Creating GLOBAL_SHAPE_INIT_SOFT');end
    global_shape = mgc_DeriveGlobalShape([inputDir '/' dirCroppedBBoxNormMedianImages '/'],paramsClassCut.xbins,paramsClassCut.ybins,paramsClassCut.obins);
    save([inputDir '/' dirCroppedBBoxNormMedianImages '/global_shape_init_soft.mat'],'global_shape');
else
    if paramsClassCut.verbose; disp('GLOBAL_SHAPE_INIT_SOFT exists');end
end


%shape
if not(exist([inputDir '/' dirCroppedBBoxNormMedianImages '/ShapePairwise.mat'],'file'))
    if paramsClassCut.verbose; disp('Creating ShapePairwise GM');end
    structTemp = load('obj.mat');
    obj = structTemp.obj;
    structTemp = load('global_shape_init_soft');
    global_shape = structTemp.global_shape;
    GM = mgc_GetPairwiseShape_reduced(obj,global_shape);
    save([inputDir '/' dirCroppedBBoxNormMedianImages '/ShapePairwise.mat'],'GM');
else
    if paramsClassCut.verbose; disp('ShapePairwise GM exists');end
end

cd(origDir);
setenv('LD_LIBRARY_PATH', org_ldlp);
