function [vector_energies segm_labels global_shape location_model_final] = classcutRunIterations(paramsClassCut)

origDir = [pwd '/'];

weights = paramsClassCut.weights;
solver = 'qpbo';

verbose = 0;

%build GMs

% load the likelihhods use to build the GMs{1} and GMs{2} in the initial iteration
cd(paramsClassCut.workingDir);
struct = load('obj_init_soft_mask_lfc');
prob_llhood_bg = struct.prob_llhood_bg; 
prob_llhood_fg = struct.prob_llhood_fg;
obj = struct.obj;
obj = classcutUpdateObj(paramsClassCut,obj);

%create GM_NULL that would be used when adding GMs
temp = mgc_UnaryNULL(obj);
GM_NULL.UE = temp.UE;
clear temp
GM_NULL.PI = uint32([1 2 1]');
GM_NULL.PE{1} = [0 0;0 0];


%within image smoothness
struct = load('GradientSmth.mat');       
GMs{3} = struct.GM;
display('GM_GradientSmth loaded');

%border
struct = load('GM_Border');
GMs{4} = struct.GM;
display('GM_Border loaded');

%area
struct = load('GM_Area');
GMs{5} = struct.GM;
display('GM_Area loaded')

%between image smoothness
weights(6) = weights(6)/length(obj);%PAIRWISE IMPACT
struct = load('LocationSmth.mat');
GMs{6} = struct.GM;
display('GM_LocationSmth loaded');
        
%location model
GMs{7} = mgc_UnaryNULL(obj); %the initial location model is uniform - so deosn't count
        
%shape model
struct = load('ShapePairwise.mat');
GMs{8} = struct.GM;
display('GM_ShapePairwise loaded');
struct = load('global_shape_init_soft');
global_shape = struct.global_shape;
%grabcut like color model
[objcopy initial_segms] = classcutGrabcutColorModelInitialization(paramsClassCut,obj);
objcopy = td_grabcutAppearance(objcopy, initial_segms);
GMs{9} = mgc_GetUnaryGrabcut(objcopy);


vector_energies = [];

flag = 1;
iter = 1;
%%
while flag  && iter < paramsClassCut.maximumIterations
    
    iter = iter+1;
    
        % 1 - build BG and FG models        
        
        GMs{1} = classcutGetUnaryGlobalAppGM(obj,prob_llhood_bg{1},prob_llhood_fg{1},1); %SURF 100
        GMs{2} = classcutGetUnaryGlobalAppGM(obj,prob_llhood_bg{2},prob_llhood_fg{2},2); %SURF 1000        
        
        % 2 - add GM coresponding to different unaries and pairwise cues          
        GM = as_CombineWeightGMs_reduced(GMs,GM_NULL,weights);
        %keyboard;
        for j=1:length(GM.UE)
            if(GM.UE{j}(1) == inf)
                display('BG INF');
            end
            if(GM.UE{j}(2) == inf)
                display('FG INF');
            end
        end
        
        % 3 run the inference and get the label of all segments
        segm_labels(iter,:) = as_SolveProblemTRW(GM,solver);
                
        energy = mgc_GetEnergy_reduced(GM,segm_labels(iter,:));
        vector_energies(end+1) = energy;
        
        obj = as_TraceBackSolutionSetSegments(obj,segm_labels(iter,:));                        
        
        obj = mgc_SetSegmentsFgBgFromSolution(obj,segm_labels(iter,:));
        
        
        if verbose
            title(['Iteration - iteration ' num2str(iter-1)]);
        end
              
        if(sum(abs(segm_labels(iter,:))) == length(segm_labels(iter,:)))%all segments are FG
            flag = 0;
        end
        if(sum(abs(segm_labels(iter,:))) == 0)%all segments are BG
            flag = 0;
        end
        
        % 4 - refine the likelihoods models using the segment histograms
        for id_cell = 1:2
            prob_llhood_bg{id_cell} = mgc_BuildModel(obj,'bg',[],id_cell,paramsClassCut.updateBG);
            prob_llhood_fg{id_cell} = mgc_BuildModel(obj,'fg',[],id_cell,paramsClassCut.updateBG);
        end
       
        if(iter > 2)            
            AA = repmat(segm_labels(iter,:),size(segm_labels,1)-1,1);
            if not(prod(sum(abs(AA-segm_labels(1:iter-1,:)),2)))
                flag = 0;
            end               
        end                
        
        %update location model
        if length(weights) >= 7
            GMs{7} = mgc_GetLocationPriorGM(obj,32,32);
        end
        
        %update the shape model
        if length(weights) >= 8            
            shape_iteration = mgc_DeriveGlobalShapeFromCurrentIter(obj,32,32,5,1);
            global_shape = (global_shape + shape_iteration)/2;
            GMs{8} = mgc_GetPairwiseShape_reduced(obj,global_shape);            
        end
        
        %update he Grabcut color like model
        if length(weights) >= 9
            obj = td_grabcutAppearance(obj);
            GMs{9} = mgc_GetUnaryGrabcut(obj);
        end                                        
end
%%
location_model_final = classcutGetLocationModelFinal(paramsClassCut,segm_labels(end,:));

%%
cd(origDir);

end