function hist = mgc_BoundaryDescriptor_bog(im, settings)


%  settings for grid
%    settings.xbins      =8  
%    settings.ybins      =8
%    settings.obins      =5

binshifts=[-1 -1;0 -1;1 -1;-1 0;0 0;1 0;-1 1;0 1;1 1];

[H W]=size(im);

mask=reshape(1:(settings.xbins*settings.ybins),[settings.ybins, settings.xbins]);
mask=imresize(mask,[H,W],'nearest');
labels=unique(rowvec(mask));
nLabels=settings.xbins*settings.ybins;


% determine the size of the individual cells of the mask. Needed for
% histogram normalization
counts=accumarray(colvec(mask+1), ones(size(colvec(mask))), [1+nLabels, 1] );
counts=counts(2:end);

if settings.softassignments
    %[H W]=size(im);
    [posY,posX]=ind2sub(size(im),find(im));%position of oriented edges
    hist=zeros(settings.ybins, settings.xbins);

    counts=(H*W)/(settings.xbins*settings.ybins)*ones(numel(hist),1);
    
    binboundariesY=linspace(1,H,settings.ybins+1); 
    bincentersY=(binboundariesY(2:end)+binboundariesY(1:end-1))/2;
    bindeltaY=binboundariesY(2)-binboundariesY(1);
    binboundariesY=binboundariesY(2:end);

    binboundariesX=linspace(1,W,settings.xbins+1); 
    bincentersX=(binboundariesX(2:end)+binboundariesX(1:end-1))/2;
    bindeltaX=binboundariesX(2)-binboundariesX(1);
    binboundariesX=binboundariesX(2:end);

    for i=1:length(posY)
        % get the current position and its value
        pos=[posY(i) posX(i)];
        va=im(pos(1), pos(2));
        
        % determine the bin into which it belongs
        bin=[find(pos(1)<=binboundariesY,1,'first'), find(pos(2)<=binboundariesX,1,'first')];
        % determine the center point of that bin
        cent=[bincentersY(bin(1)), bincentersX(bin(2))];
             
        % determine the neighboring bins and their center points 
        neighborBins=repmat(bin,[9 1])+binshifts;
        neighborCents=repmat(cent,[9 1])+repmat([bindeltaY bindeltaX],[9 1]).*binshifts;
        
        indexBinsInside = find(prod(neighborBins,2) > 0); %at least 4 are inside
        neighborBins = neighborBins(indexBinsInside,:);
        neighborCents = neighborCents(indexBinsInside,:);
        % compute the distances to the surrounding bin centers
        dists = neighborCents-repmat(pos, [size(neighborCents,1) 1]);   
        dists = sum(dists.*dists,2);
        % make the central bin a bit closer (to rule out ties)
        %dists(5)=dists(5)-eps;
        % make the probabilities
        dists=exp(-dists);
        [dists ix]=sort(dists,'descend');
        
        % norm the probabilities so that the first four are the same
        dists=dists/sum(dists(1:4)); % the first four bins should have a total mass of 1.0 times the actual value
        %fprintf(1,'Assigning to bins: ')
        for j=1:4
            bin=neighborBins(ix(j),:);
            %fprintf(' (%i,%i)',bin(1),bin(2));
            if all(bin>=[1 1] & bin<=[settings.ybins settings.xbins])
                hist(bin(1),bin(2))=hist(bin(1),bin(2))+dists(j)*va;
            else % if we are on a border bin, put everything that goes outside to the original one
                hist(neighborBins(ix(1),1),neighborBins(ix(1),2))=hist(neighborBins(ix(1),1),neighborBins(ix(1),2))+dists(j)*va;
            end
        end
        %fprintf(1,'\n');
    end
    
%     %  old, slow method
%     smoothingradius=sqrt(sum(rowvec(mask==1)));
%     smoothingfilter=fspecial('gaussian',3*ceil(smoothingradius),smoothingradius);
%     
%     
%     for c=1:(settings.xbins*settings.ybins)
%         %cmask=conv2(double(mask==c),smoothingfilter,'same');
%         cmask=imfilter(double(mask==c),smoothingfilter,'same');
%         hist(c)=sum(rowvec(cmask.*im));
%     end


    hist=colvec(hist);

else
    
    % compute the boundary multiplied with the mask
    lpshape=im.*mask;
    hist=accumarray(colvec(lpshape+1), ones(size(colvec(lpshape))), [1+nLabels, 1] );
    hist=hist(2:end);    
end 

hist=hist./counts;
hist(isnan(hist))=0;
