function hist_global = mgc_DeriveGlobalShape(input_dir,xbins,ybins,obins,verbose)

org_dir = pwd;
org_ldlp = getenv('LD_LIBRARY_PATH');
setenv('LD_LIBRARY_PATH', '');


if nargin < 5
    verbose = 0;
end

cd(input_dir);
segm = dir('*.ppm');


for idx = 1:length(segm)
    splist(idx).filename = [input_dir '/' segm(idx).name];
    %idx
end
splist=td_loadImages(splist);

settings=vgg_argparse(struct(...
    'mask',2,...    % 1=grid, 2=logpolar
    'cropMinimal',0,...   % 0=assume normalized bounding boxes, 1=normalize wrt to the segmentation, 
    'rBins',6,...         % radius bins for log polar coordingates
    'aBins',8,...         % angular bins for log polar coordingates
    'xbins',32,...         % x/y bins for the grid coordinates
    'ybins',32,...
    'obins',5,...
    'softassignments',1));

if nargin >= 2 settings.xbins = xbins; end
if nargin >= 3 settings.ybins = ybins; end
if nargin >= 4 settings.obins = obins; end

xbins = settings.xbins;
ybins = settings.ybins;
obins = settings.obins;

%% compute boundary for all best possible segmentations and accumulate in
%% the grid edgels
hist_global = zeros(settings.ybins*settings.xbins,settings.obins);

for n=1:length(splist)        
    %n
    segments=double(splist(n).img);
    % put it into a single value image by 256ary addition
    segments(:,:,1)=segments(:,:,1)*256*256;
    segments(:,:,2)=segments(:,:,2)*256;
    segments=sum(segments,3);
    % now make the labels 1:L
    segLabels=unique(rowvec(segments));
    s=zeros(size(segments));
    cnt=1;
    for l=segLabels
        s(segments==l)=cnt;
        cnt=cnt+1;
    end
    splist(n).img=s;
    
    [~, img_boundary] = mgc_GetBoundaryFromSegments(splist(n).img,verbose);
    splist(n).img_boundary = img_boundary;
    [dx dy] = gradient(img_boundary);
    mag = sqrt(dx.^2 + dy.^2);
    phi = atan2(dy,dx);
    phi(phi<0) = phi(phi<0) + pi;
    
    %    %hard assignment
    %     for orientation = 1:settings.obins
    %         img = zeros(size(img_boundary));
    %         idx = (phi >= (orientation - 1) *  pi/obins) .* (phi <= orientation  * pi/obins);
    %         index = find(idx == 1);
    %         img(index) = img_boundary(index);
    %         splist(n).hist(:,orientation) = mgc_BoundaryDescriptor(img, settings);
    %     end
    
    [orientations trash1 trash2] = mgc_splitOrientations(mag,phi,0.1,false,uint32(settings.obins));
    for orientation = 1:settings.obins
        img=orientations(:,:,orientation);
        %splist(n).hist(:,orientation) = mgc_BoundaryDescriptor(img, settings);
        splist(n).hist(:,orientation) = mgc_BoundaryDescriptor_bog(img, settings);%modified by bogdan
    end
    hist_global = hist_global + splist(n).hist;
    
    if verbose
        pause;
        close gcf;
    end
end
hist_global = reshape(hist_global,[settings.ybins settings.xbins settings.obins]);
hist_global = hist_global/n;

% for orientation = 1:settings.obins
%     %figure,imshow(hist_global(:,:,orientation))
%     figure,imagesc(hist_global(:,:,orientation))
% end

cd(org_dir);
setenv('LD_LIBRARY_PATH', org_ldlp);



