function hist_global = mgc_DeriveGlobalShapeFromCurrentIter(obj,xbins,ybins,obins,softassignments,verbose)

if nargin < 6
    verbose =0;
end

settings=vgg_argparse(struct(...
    'mask',2,...    % 1=grid, 2=logpolar
    'cropMinimal',0,...   % 0=assume normalized bounding boxes, 1=normalize wrt to the segmentation, 
    'rBins',6,...         % radius bins for log polar coordingates
    'aBins',8,...         % angular bins for log polar coordingates
    'xbins',32,...         % x/y bins for the grid coordinates
    'ybins',32,...
    'obins',5,...
    'softassignments',1));

if nargin >= 2 settings.xbins = xbins; end
if nargin >= 3 settings.ybins = ybins; end
if nargin >= 4 settings.obins = obins; end
if nargin >= 5 settings.softassignments = softassignments; end

xbins = settings.xbins;
ybins = settings.ybins;
obins = settings.obins;

hist_global = zeros(xbins*ybins,obins);

for img_id = 1:length(obj)
    
    img_reconstructed = mgc_DeriveImgFromSetSegms(obj(img_id),'fg');
    [~, img_boundary] = mgc_GetBoundaryFromSegments(img_reconstructed);
    [dx dy] = gradient(img_boundary);
    mag =sqrt(dx.^2 + dy.^2);
    phi = atan2(dy,dx);
    phi(phi<0) = phi(phi<0) + pi;
    
    hist =[];
    %hard assignment
    
%     for orientation = 1:obins
%         img = zeros(size(img_boundary));
%         idx = (phi >= (orientation - 1) *  pi/obins) .* (phi <= orientation  * pi/obins);
%         index = find(idx == 1);
%         img(index) = img_boundary(index);
%         hist(:,orientation) = mgc_BoundaryDescriptor(img, settings);
%     end

    [orientations trash1 trash2] = mgc_splitOrientations(mag,phi,0.1,false,uint32(settings.obins));
    for orientation = 1:settings.obins
        img=orientations(:,:,orientation);
        %splist(n).hist(:,orientation) = mgc_BoundaryDescriptor(img, settings);
        hist(:,orientation) = mgc_BoundaryDescriptor_bog(img, settings);%modified by bogdan
    end   
    hist_global = hist_global + hist;    
    
end
hist_global = reshape(hist_global,[settings.ybins settings.xbins settings.obins]);
hist_global = hist_global/img_id;

if verbose
    for orientation = 1:settings.obins
        figure,imagesc(hist_global(:,:,orientation))
    end
end