% GM=GMadd(gm1, w1, gm2, w2)
%  - calculate the weighted sum of two Graphical Models
%  - if gm2 is not complete (i.e. either the unary or the pairwise term is missing, 
%    only the respective other is treated


function GM=mgc_GMadd_bog_reduced_GM(gm1,w1,gm2,w2,gm_null)

%gm1 = mgc_CheckUE(gm1);
%gm2 = mgc_CheckUE(gm2); 
GM=gm1;

if ~isfield(GM,'UE') && isfield(gm2,'UE')
    if(w2==1)
        GM.UE = gm2.UE;
    else
        if(w2==0)
            GM.UE = gm_null.UE;
        else
            for i=1:size(gm2.UE,2)
                GM.UE{i}=w2*gm2.UE{i};    
            end
        end
    end
    %GM=scaleUnary(GM,w2);
else
    if isfield(gm2,'UE')
        %size(gm1.UE),size(gm2.UE)
        if prod(double(size(gm1.UE)==size(gm2.UE)))
            for i=1:size(gm1.UE,2)
                GM.UE{i}=w1*gm1.UE{i}+w2*gm2.UE{i};
            end
        else
            error('GMs do not match when adding unary terms!\n');
        end
    else
        %fprintf(1,'Ignoring UNARY\n');
    end
end

if ~isfield(GM,'PE') && isfield(gm2,'PE')    
    GM.PI=gm2.PI;  
    if(w2==1)
        GM.PE = gm2.PE;
    else
        if(w2==0)
            GM.PE = gm_null.PE;
            GM.PI = gm_null.PI;
        else
            for i=1:size(gm2.PE,2)
                GM.PE{i}=w2*gm2.PE{i};    
            end
        end
    end
    %GM=scalePairwise(GM,w2);
else
    if isfield(gm2,'PE')
        
        gm_aux.PE = cell(1,size(GM.PE,2)+size(gm2.PE,2));
        gm_aux.PI = zeros(3,size(GM.PE,2)+size(gm2.PE,2));
        gm_aux.PE(1:size(GM.PE,2)) = GM.PE;
        gm_aux.PI(:,1:size(GM.PE,2)) = GM.PI;
        
        visited_gm2 = zeros(1,size(gm2.PI,2));
        matches = 0;
        %check correspondence terms from gm1.PI
%         tic
%         %very expensive
%         for idc1 = 1:size(gm1.PI,2) %id column
%             pi1_column = GM.PI(:,idc1);
%             for idc2 = 1:size(gm2.PI,2)
%                 pi2_column = gm2.PI(:,idc2);
%                 %matching condition first 2 component are equal 
%                 if isequal(pi1_column(1:2),pi2_column(1:2))
%                     %update GM.PE
%                     gm_aux.PE{pi1_column(3)} = w1*gm1.PE{pi1_column(3)}+w2*gm2.PE{pi2_column(3)};
%                     visited_gm2(pi2_column(3)) = 1;
%                     matches = matches+1;
%                     break;
%                 end
%             end
%         end
%         
%         toc
%         index = size(GM.PE,2);
%         %chech column form gm2 which wew not matched
%         for idc2 = 1:size(gm2.PI,2)
%             if not(visited_gm2(idc2))
%                 index = index + 1;
%                 gm_aux.PI(:,index) = gm2.PI(:,idc2);
%                 gm_aux.PE{index} = gm2.PE{idc2};
%             end
%         end        
%         GM.PE = gm_aux.PE(1:index);
%         GM.PI = gm_aux.PI(1:index);
                
        base = length(GM.UE)+1;
        base_vector1 = [base 1] *double(GM.PI(1:2,:));%sorted vector
        base_vector2 = [base 1] *double(gm2.PI(1:2,:));%sorted_vector
        index1 =1;
        index2 =1;
        for idc1 = 1:size(GM.PI,2)
            %pi1_column = GM.PI(:,idc1);
            if(base_vector1(index1) == base_vector2(index2))                
                gm_aux.PE{index1} = w1*gm1.PE{index1}+w2*gm2.PE{index2};
                visited_gm2(index2) = 1;
                matches = matches+1;
                index1 = index1 + 1;
                index2 = index2 + 1;
                if (index1 > length(base_vector1)) || (index2 > length(base_vector2))
                    break;
                end
            else
                aux = (base_vector1(index1) > base_vector2(index2));
                index1 = index1 + aux;
                index2 = index2 + aux;   
                if (index1 > length(base_vector1)) || (index2 > length(base_vector2))
                    break;
                end
            end
        end        
        index = size(GM.PE,2);
        %check column from gm2 which was not matched
        for idc2 = 1:size(gm2.PI,2)
            if not(visited_gm2(idc2))
                index = index + 1;
                %gm_aux.PI(:,index) = gm2.PI(:,idc2); -CVPR2010 submission
                gm_aux.PI(1:2,index) = gm2.PI(1:2,idc2);
                gm_aux.PI(3,index) = index;                
                gm_aux.PE{index} = w2*gm2.PE{idc2};
            end
        end
        GM.PE = gm_aux.PE(1:index);
        GM.PI = gm_aux.PI(:,1:index);
        
%         if prod(prod(double(gm1.PI==gm2.PI))) && prod(double(size(gm1.PE)==size(gm2.PE)))
%             for i=1:size(gm1.PE,2)
%                 GM.PE{i}=w1*gm1.PE{i}+w2*gm2.PE{i};
%             end
%         else
%             fprintf(1,'GMs do not match when adding pairwise terms\n')
%         end
    else
        %fprintf(1,'Ignoring PAIRWISE\n');
    end
end

if ~isfield(gm2,'state')
    gm2.state='UNK';
end

if ~isfield(gm1,'state')
    gm1.state='UNK';
end
    
GM.PI = uint32(GM.PI);
GM.state=strcat('ADD(',num2str(w1),'*',gm1.state,'+',num2str(w2),'*',gm2.state,')');
