function [GM location_model] = mgc_GetLocationPriorGM(obj,xbins,ybins,verbose)


if nargin < 4
    verbose = 0;
end

nb_images = length(obj);
img_segms = zeros(1, nb_images);

for i = 1: nb_images
    img_segms(i) = length(obj(i).segms);
end

total_segms = sum(img_segms);

GM.state = 'Location prior - unary';
GM.UE = cell(1,total_segms);

settings=vgg_argparse(struct(...
    'mask',2,...    % 1=grid, 2=logpolar
    'cropMinimal',0,...   % 0=assume normalized bounding boxes, 1=normalize wrt to the segmentation, 
    'rBins',6,...         % radius bins for log polar coordingates
    'aBins',8,...         % angular bins for log polar coordingates
    'xbins',32,...         % x/y bins for the grid coordinates
    'ybins',32,...
    'obins',5,...
    'softassignments',1));

if nargin >= 2 settings.xbins = xbins; end
if nargin >= 3 settings.ybins = ybins; end

xbins = settings.xbins;
ybins = settings.ybins;

hist = zeros(xbins*ybins);

pos=[];   
neg=[];

for img_id = 1:length(obj)
                
    segmentation_id = mgc_DeriveImgFromSetSegms(obj(img_id),'fg');    
       
    %hist = mgc_BoundaryDescriptor(segmentation_id,settings);
    hist = td_shapeDescriptor(segmentation_id, settings);%much faster
    if isempty(pos)
        pos=hist;
        neg=1-hist;
    else
        pos=pos+hist;
        neg=neg+1-hist;
    end
    
end

pos=pos/length(obj);
neg=neg/length(obj);
   
pos = pos + eps;
neg = neg + eps;
idx_segment_obj = 0;
for idx=1:length(obj)
    
    s = double(obj(idx).segm_labels);
   
    % get the shape descriptor mask fitting the current image
    mask=td_getShapeDescriptorMask(size(s,1), size(s,2),settings);
    
    % now replace the values in the mask by the scores log-ratio of pos/neg
    scoremask=zeros(size(mask));
    for n=1:size(mask,1)
        for m=1:size(mask,2)
            idx_mask=mask(n,m);
            scoremask(n,m)=log(pos(idx_mask)/neg(idx_mask));
        end
    end    
    
    if verbose
        figure(1)
        imagesc(scoremask);
    end
    %% HERE ARE THE SCORES: can be used as a unary
    scores=accumarray(colvec(s),colvec(scoremask),[length(unique(rowvec(s))),1]);
    counts = zeros(max(max(s)),1);
    for jj=1:max(max(s))
        counts(jj) = length(find(s == jj));        
    end
    scores = scores./counts;
    
    
    for sid_segm = 1:img_segms(idx)
        idx_segment_obj = idx_segment_obj + 1;         
        GM.UE{idx_segment_obj} = single(- scores(sid_segm) * [0 1]);        
    end
    % now show the superpixels with their scores
    scoredSPs=zeros(size(s));
    for n=1:size(s,1)
        for m=1:size(s,2)
            scoredSPs(n,m)=scores(s(n,m));
        end
    end     
    if verbose
        figure(2)
        imagesc(scoredSPs);
        rowvec(unique(rowvec(mask)))
        pause
    end        
end
location_model = scoremask;

