function obj=td_grabcutAppearance(obj, segs, settings)
% input obj struct
% input (optional): Segs (cell array of segmentations for the images)
% output obj struct with additional energies for each superpixel on
% foreground an background
%
%
% settings.shrink=0; -> no shrinking of the regions
%                 1; -> shrink with erode with a disk of size
%                 settings.shrinkpar
% settings.shrinkpar=10; meaning depends on settings.shrink
% settings.nmixtures=5; -> number of mixtures of FG and BG model
% settings.unclearThreshold=0 -> I have no clue what it means, it come
%          somewhere out of grabcut and value 0 looks best on visualizing the
%          energies


if nargin<1
    load /home/bogdan/Desktop/Caltech101_bonsai_full_images_obj_after_1_iter.mat
end

if nargin<2
    segs={};
end

if nargin<3
    settings.shrink=0;
    settings.nmixtures=5;
    settings.shrinkpar=10;
    settings.unclearThreshold=0;
end


FGpixels=[];
BGpixels=[];

fprintf(1,'Gathering appearance information about the images\n');
for n=1:length(obj)
    obj(n).img=double(td_imreadc(obj(n).name))/255;    
    %figure,imshow(obj(n).img);keyboard;
    if isempty(segs)
        FG = mgc_DeriveImgFromSetSegms(obj(n),'fg');
        BG = mgc_DeriveImgFromSetSegms(obj(n),'bg');
    else
        FG=segs{n};
        BG=segs{n}==0;
    end
    
    if settings.shrink==0
    elseif settings.shrink==1
        FG=imerode(FG,strel('disk',settings.shrinkpar));
        BG=imerode(BG,strel('disk',settings.shrinkpar));
    else
        error('unknown shrink parameter');
    end
    
    FGpixels=[FGpixels ; nema_impixel(obj(n).img, find(FG))];
    BGpixels=[BGpixels ; nema_impixel(obj(n).img, find(BG))];    
end

fprintf(1,'Estimating appearance model\n');
fgGMM=namg_init_gmm(FGpixels', settings.nmixtures);
bgGMM=namg_init_gmm(BGpixels', settings.nmixtures);

fprintf(1,'Computing unary terms\n');
for n=1:length(obj)
    kf = namg_assign_gmm( obj(n).img, fgGMM, 128*ones(size(obj(n).img(:,:,1))) );
    kb = namg_assign_gmm( obj(n).img, bgGMM, 128*ones(size(obj(n).img(:,:,1))) );
    STE=namg_get_STE_gmm(obj(n).img,128*ones(size(obj(n).img(:,:,1))) , fgGMM, bgGMM, kf, kb, settings.unclearThreshold);
    scores=reshape(STE',[size(obj(n).img(:,:,1)),2]);
    
    
    %% THIS IS BECAUSE WE WANT TO BE ABLE TO COMPUTE ON GRAY LEVEL IMAGES
    %% AND UNFORTUNATELY THE COVARIANCE OF THE MIXTURES IS IN-INVERSIBLE
    %% FOR THESE STUPID IMAGES
    scores(isnan(scores))=0;
    
    fgScores=scores(:,:,1);
    bgScores=scores(:,:,2);
    siz=size(fgScores);    

    for k=1:length(obj(n).segms)
        % this can be made faster using sub2ind
        fgScore=0;
        bgScore=0;
        nPix=size(obj(n).segms(k).coords,2);
        for i=1:nPix
            x=obj(n).segms(k).coords(1,i);
            y=obj(n).segms(k).coords(2,i);
            fgScore=fgScore+fgScores(y,x);
            bgScore=bgScore+bgScores(y,x);
        end
        
        obj(n).segms(k).grabcutFgAppScore=fgScore/nPix;
        obj(n).segms(k).grabcutBgAppScore=bgScore/nPix;
    end
end

