function hist=td_shapeDescriptor(im, settings)

% settings with their default values (if not specified)
%
% settings.mask       =1   logpolar / 2=grid
%
% settings.cropMinimal=0  the entire image is the coordinate system
%                         1=crop the image to the minimal bounding box
%
%  settings for logpolar
%    settings.rBins      =6   number of radius bins
%    settings.aBins      =8   number of angle bins
%  settings for grid
%    settings.xbins      =7  
%    settings.ybins      =7



if nargin<2
    settings.mask=1;
    settings.rBins=6;
    settings.aBins=8;
end


if settings.cropMinimal==1
    tmp=regionprops(im,'FilledImage');
    im=tmp.FilledImage;
end

[H W]=size(im);

[mask, labels, nLabels]=td_getShapeDescriptorMask(H,W,settings);

% determine the size of the individual cells of the mask. Needed for
% histgram normalization
counts=accumarray(colvec(mask+1), ones(size(colvec(mask))), [1+nLabels, 1] );

% compute the shape multiplied with the mask
lpshape=im.*mask;

hist=accumarray(colvec(lpshape+1), ones(size(colvec(lpshape))), [1+nLabels, 1] );

hist=hist./counts;
hist=hist(2:end);
hist(isnan(hist))=0;


