function I = AdjacentSegments(I, adj_type, verbose)

% add field I(ix).(adj_type)(i,j) = 0|1 whether I(ix).segms(i)
% is adjacent to I(ix).segms(j).
%

% if exact adjacency is requested, 2 pixels are adjacent when their distance is exactly 1.
% In order to be tolerant to Felzenswalb's segmentor producing very thin
% cushion regions between truly adjacent segments, the 'near_adjacent' mode
% allows up to 8 pixels distance.
switch adj_type
  case 'adjacent'
    pthresh = 1; min_shared = 0.02;  % 2% of boundary pixel must be adjacent for the segments to be counted as adjacent (both ways)
    shared_type = 2;                 % both segments must share at least min_shared%
  case 'near_adjacent'
    pthresh = 8; min_shared = 0.02;
    shared_type = 2;
  case 'heavily_adjacent'
    pthresh = 1; min_shared = 0.1;   % 10% of boundary pixel must be adjacent for the segments to be counted as adjacent (both ways)
    shared_type = 1;                 % at least one of the segments must share at least min_shared% (allow for a segment inside another to be heavily adjacent)
otherwise
  error([mfilename ': unknown adjacency type ' adj_type]);
end


if nargin < 3
  verbose = false;
end

for ix = 1:length(I)
  display(['Image ' num2str(ix)]);
  segms = I(ix).segms;
  I(ix).(adj_type) = false(length(segms),length(segms));  % logical array -> spare memory !
  for six1 = 1:length(segms)
    for six2 = (six1+1):length(segms);
      
        
      s1 = segms(:,six1);
      s2 = segms(:,six2);
      %
      % quick reject test:
      % BBs +1 pixel overlap ?
      bb1 = double(s1.bb);
      bb2 = double(s2.bb);
      bb1 = bb1 + [-2 +2; -2 +2];
      bb2 = bb2 + [-2 +2; -2 +2];
      if doBBoverlap(bb1, bb2)
        % points on boundary of the two segments
        C1 = s1.perim_coords;
        C2 = s2.perim_coords;
        %
        [trash dists2] = vgg_nearest_neighbour(double(C1(:,1:10:end)), double(C2(:,1:10:end)));
        if min(dists2) <= 10^2                 % precondition to speedup things !
          [trash dists2] = vgg_nearest_neighbour(double(C1), double(C2));
          adj_pts = sum(dists2 <= pthresh^2);      
          % min_shared% of boundary pixel must be adjacent for the segments to be counted as adjacent (both ways)
          if shared_type == 2
            meet = ((adj_pts / size(C1,2)) > min_shared) && ((adj_pts / size(C2,2)) > min_shared);
          else
            meet = ((adj_pts / size(C1,2)) > min_shared) || ((adj_pts / size(C2,2)) > min_shared);
          end
        else
          meet = false;
        end 
      else
        % failed quick reject test
        meet = false;
      end  % do BBs overlap ?
      %
      % set output
      I(ix).(adj_type)(six1,six2) = meet;
      I(ix).(adj_type)(six2,six1) = meet;
      %
      % visualiz
      if verbose
        figure; hold on;
        C1 = s1.perim_coords;
        C2 = s2.perim_coords;
        plot(C1(1,:), C1(2,:), 'bo');
        plot(C2(1,:), C2(2,:), 'ro');
        axis ij; axis equal;
        disp(['segments ' num2str(six1) ', ' num2str(six2) ' -> adjacent = ' num2str(meet)]);
        keyboard;
      end
      
      
    end % loop over s2
  end % loop over s1
end % loop over images
