% copied out of nema_matte_grabcut for external use

function k = namg_assign_gmm( C, gmm, T )
%% Assign pixels to a Gaussian from a GMM.
% 
% Input parameters:
%   C - Input image with same dimensionality as the gmm structure.
%   gmm - Gaussian mixture model.
%   T - Trimap image.
%
% Output paramters:
%   k - Gaussian assignments.

[ nrows, ncols, ndims ] = size( C );
if length( gmm.pi ) < 256
  k = zeros( nrows, ncols, 'uint8' );
else
  error( 'The number of clusters exceeds 256' );
end

indices = find( T == 128 | T == 64 | T == 192 );
pixels = nema_impixel( C, indices )';
npixels = length( indices ); %nrows * ncols;
logp = zeros( length( gmm.pi ), npixels );

for i = 1:length( gmm.pi )
  logp( i, : ) = log( gmm.pi( i ) ) + ...
      nema_lognorm( pixels, gmm.mu( :, i ), gmm.sigma( :, :, i ) );
end

[maxp, maxpi] = max( logp, [], 1 );
k( indices ) = maxpi;
